"""
Freezing-point functions.
"""

from ._wrapped_ufuncs import (
    CT_freezing,
    CT_freezing_first_derivatives,
    CT_freezing_first_derivatives_poly,
    CT_freezing_poly,
    SA_freezing_from_CT,
    SA_freezing_from_CT_poly,
    SA_freezing_from_t,
    SA_freezing_from_t_poly,
    pot_enthalpy_ice_freezing,
    pot_enthalpy_ice_freezing_first_derivatives,
    pot_enthalpy_ice_freezing_first_derivatives_poly,
    pot_enthalpy_ice_freezing_poly,
    pressure_freezing_CT,
    t_freezing,
    t_freezing_first_derivatives,
    t_freezing_first_derivatives_poly,
)
